import 'dart:convert';

import 'package:crypto/crypto.dart';
import 'package:flutter/material.dart';
import 'package:http/http.dart';
import 'package:kucoin_api/src/models/account.dart';

class KuCoinApi {
  final String basePath;
  String apiVersion;

  Map<String, String> initialHeaders = <String, String>{};
  Client httpClient = Client();
  final String _apiKey;
  final String _secretKey;
  final String _passphrase;
  final String _webSocketBaseAddress = '';
  KuCoinApi(this._apiKey, this._secretKey, this._passphrase,
      {this.basePath = "https://api.kucoin.com", this.apiVersion = '2'});

  Future<List<Account>> getAccounts() async {
    final response = await get('accounts', initialHeaders);
    return json
        .decode(response.body)
        .map((i) => Account.fromJson(i))
        .toList(growable: false);
  }

  String describeEnum(Object enumEntry) {
    final String description = enumEntry.toString();
    final int indexOfDot = description.indexOf('.');
    assert(
      indexOfDot != -1 && indexOfDot < description.length - 1,
      'The provided object "$enumEntry" is not an enum.',
    );
    return description.substring(indexOfDot + 1);
  }

  Uri _getFullPath(String path) => Uri.parse('$basePath/api/v1/$path');

  Future<Response> get(String path, Map<String, String> headers,
      {bool signed = true}) async {
    var uri = _getFullPath(path);
    var signedHeaders = _getSignedHeaders(headers, uri, signed);

    Response res;
    if (signedHeaders.isNotEmpty) {
      res = await httpClient.get(uri, headers: signedHeaders);
    } else {
      res = await httpClient.get(uri);
    }

    return res;
  }

  Future<Response> _post(
    String path,
    Map<String, String> headers, {
    Map<String, String>? body,
    bool signed = true,
  }) async {
    var uri = _getFullPath(path);
    var res = await httpClient.post(uri,
        body: body, headers: _getSignedHeaders(headers, uri, signed, body));

    return res;
  }

  Map<String, String> _getSignedHeaders(
      Map<String, String> headers, Uri uri, bool signed,
      [Map<String, String>? body]) {
    if (signed) {
      if (headers.isEmpty) headers = <String, String>{};
      headers.addAll({
        'KC-API-KEY': _apiKey,
        'KC-API-SIGN': _sign(uri, body),
        'KC-API-TIMESTAMP': DateTime.now().millisecondsSinceEpoch.toString(),
        'KC-API-PASSPHRASE': _passphrase,
        'KC-API-KEY-VERSION': apiVersion
      });
    }
    return headers;
  }

  @protected
  String _sign(Uri uri, [Map<String, String>? body, String? secretKey]) {
    if (body != null && body.isNotEmpty) {
      uri = Uri(
        scheme: uri.scheme,
        fragment: uri.fragment,
        host: uri.host,
        path: uri.path,
        port: uri.port,
        query: uri.query,
        userInfo: uri.userInfo,
        queryParameters: body,
      );
    }

    var key = utf8.encode(secretKey ?? _secretKey);
    var toSign = uri.path;
    if (uri.hasQuery) toSign += '?' + uri.query;
    var bytes = utf8.encode(toSign);

    var hmacSha256 = Hmac(sha256, key); // HMAC-SHA256
    var digest = hmacSha256.convert(bytes);

    return digest.toString();
  }
}
